package com.ca.chorus.experiement;

import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import org.teiid.translator.DataNotAvailableException;
import org.teiid.translator.ExecutionContext;
import org.teiid.translator.ResultSetExecution;
import org.teiid.translator.TranslatorException;

import com.google.common.eventbus.Subscribe;

public class ContinuousExecutionHandlingDelegateExecution<F, C>
    implements
        ResultSetExecution
{
    private final ResultSetExecution                    resultSetExecution;

    private final org.teiid.translator.ExecutionContext executionContext;

    private final AtomicBoolean                         endOfResultReached;

    private final MessageBus                            messageBus;

    public ContinuousExecutionHandlingDelegateExecution(final MessageBus messageBus,
                                                        final ExecutionContext executionContext,
                                                        final AtomicBoolean endOfResultReached,
                                                        final ResultSetExecution resultSetExecution)
    {
        this.messageBus = messageBus;
        this.executionContext = executionContext;
        this.endOfResultReached = endOfResultReached;
        this.resultSetExecution = resultSetExecution;
        this.messageBus.register(this);
    }

    public void close()
    {
        this.resultSetExecution.close();
        cleanUp();
    }

    private void cleanUp()
    {
        this.messageBus.unregister(this);
    }

    public void cancel() throws TranslatorException
    {
        this.resultSetExecution.cancel();
        cleanUp();
    }

    public void execute() throws TranslatorException
    {
        if (this.endOfResultReached.compareAndSet(true, false))
        {
            throw DataNotAvailableException.NO_POLLING;
        }
        this.resultSetExecution.execute();
    }

    public List<?> next() throws TranslatorException, DataNotAvailableException
    {
        final List<?> next = this.resultSetExecution.next();
        if (next == null)
        {
            this.endOfResultReached.set(true);
        }
        return next;
    }

    @Subscribe
    public void onDataChange(final DataSetChangedEvent dataSetChangedEvent)
    {
        this.executionContext.dataAvailable();
    }
}
